pro mk_synsfc_coeff, index_in, sfcs_in, $
                     out_file=out_file, sigma=sigma, ep_date=ep_date, boxnote=boxnote
;+
;NAME:
;         mk_synsfc_coeff.pro
;
;PURPOSE: 
;         Calculate a 15x512x512 data structure of coefficients 
;         derived from a per-pixel fit of terms containing 
;         x, y, roll, solar_diameter.
;
;CALLING SEQUENCE:
;         mk_synsfc_coeff, index_in, sfcs_in
;         mk_synsfc_coeff, index_in, sfcs_in, sigma=sigma
;         mk_synsfc_coeff, index_in, sfcs_in, sigma=sigma, ep_date = '12-dec-99' 
;
;INPUT:   
;         index_in - A vector the index records of the SFC images in sfcs_in.
;         sfcs_in -  Cube of sfc images corresponding to index_in records,
;                    and used as the data-set to calculate the synthetic SFC
;                    coefficient data structure.
;
;OPTIONAL KEYWORD INPUT:
;         ep_date - specify the date to be used to calculate epoch under
;                   consideration.  Default: date of first index record.
;                   in index_in.
;
;         coeff_file - Name of output coefficient file.  Default is
;                      anticipated by mk_syn_sfc.pro.
;
;         boxnote    - String Description of 'box' filter for permissable SFCs.
;
;OUTPUT:  sigma - keyword sigma output from idl regress function.
;         ep_date - 
;
;History: pgs: Apr 10, 2000
;         9-Jan-2000 - S.L.Freeland - enhance epoch check and file name
;         2001-Jul-19 - T. Metcalf - Added code to find a bounding surface
;                                    for all the data points.  This will be
;                                    used to decide if a synthetic SFC is
;                                    interpolated or extrapolated.
;         2001-July-20 - T. Metcalf - Added a new weighting scheme for the
;                                     data in the fits.  Highly clustered
;                                     data is weighted less than sparse data.
;-

IF (n_elements(index_in) LT 30) THEN print, $
    'WARNING: Fitting 14 parameters with less than 30 data points.  Statistical Fit May be Poor.' 

;Correct processing requires all SFC images in sfcs_in to be 512x512. 
res = gt_res(index_in)
dummy_var=where(res ne 1., n_bad_res)
IF ( n_bad_res  gt 0 ) THEN $
   MESSAGE, 'All elements of sfcs_in must be 512x512.'
num_X_pixels = (size(sfcs_in(*,*,0)))(1)
num_Y_pixels = (size(sfcs_in(*,*,0)))(2)

;Build (num_X_pixels)x(num_y_pixels)x15-d data structure for 'fit' coefficients
matrix=replicate({coeff_vect:fltarr(num_X_pixels,num_Y_pixels)},15) ;

;Get vectors of parameters relavent to synthetic SFC fitting/construction.
xy_param=sxt_cen(index_in, roll=roll)
x_param=xy_param(0,*)
y_param=xy_param(1,*)
roll_param=roll
solarD_param= (get_sun(index_in))(1,*)

;Build 14-d data structure to hold relavent params for each index_in entry.
indep_vector = fltarr(14,N_ELEMENTS(index_in))
      
;Initialize the vector of 14-d params 
FOR m=0, N_ELEMENTS(index_in)-1 DO BEGIN
   indep_vector( 0,m) = x_param(m) 
   indep_vector( 1,m) = y_param(m)
   indep_vector( 2,m) = roll_param(m)
   indep_vector( 3,m) = solarD_param(m)
   indep_vector( 4,m) = (x_param(m))^2
   indep_vector( 5,m) = (y_param(m))^2
   indep_vector( 6,m) = (roll_param(m))^2
   indep_vector( 7,m) = (solarD_param(m))^2
   indep_vector( 8,m) = (x_param(m))*(y_param(m))
   indep_vector( 9,m) = (x_param(m))*(roll_param(m))
   indep_vector(10,m) = (y_param(m))*(roll_param(m))
   indep_vector(11,m) = (x_param(m))*(solarD_param(m))
   indep_vector(12,m) = (y_param(m))*(solarD_param(m))
   indep_vector(13,m) = (roll_param(m))*(solarD_param(m))
ENDFOR

; A pointer to the parts of indep_vector which give the fundamental
; independent parameters (i.e. x but not x^2 or y*x etc.)
ssindep = [0,1,2,3]

;Generate the simplex that bounds the dataset.  When interpolating later,
;we can check if the point is inside or outside the surface.  If inside we
;are interpolating (good) and if outside we are extrapolating (bad).  The
;simplex is then stored with the coefficients.

if 1 then begin
   box_message,'Inhibiting SIMPLEX calculation'
   simplex=0
endif else begin 
volume = 0.0
for i=0,5 do begin   ; Repeat a few times to get the best surface
   s = simplex_bound(indep_vector[ssindep,*],v,verbose=0)
   if (i EQ 0) OR (v LT volume) then begin
      simplex=s
      volume=v
   endif
endfor
if volume GT 5.0 then $
   message,/info,strcompress('Warning: Simplex volume is above 5 ('+ $
            string(volume)+ $
            ').  You might want to run this again to get a better surface.')

endelse
;Required by regress.pro
;weights      = REPLICATE(1.0, N_ELEMENTS(index_in))
weights      = REPLICATE(0.0, N_ELEMENTS(index_in))
z = indep_vector[ssindep,*]  ; A dimensionless measure of the data locations
for i=0L,n_elements(z[*,0])-1L do z[i,*] = (z[i,*]-mean(z[i,*]))/stdev(z[i,*])
for i=0L,N_ELEMENTS(index_in)-1 do begin
   for j=0L,N_ELEMENTS(index_in)-1 do begin
      ; Sum the distance to all other data points.  This weights will 
      ; weight the data in the fit kind of like the inverse of the density 
      ; of the data points in the indep_vector space.
      weights[i] = weights[i] + (sqrt(total((z[*,j]-z[*,i])^2)))
   endfor
endfor
totweights = total(weights)
if totweights LE 0.0 then weights = REPLICATE(1.0, N_ELEMENTS(index_in)) $
else weights = weights/totweights
;plot,z[0,*],z[1,*],psym=1  ; plot x-y points and their weight
;plots,z[0,*],z[1,*],psym=1,color=bytscl(weights/min(weights),top=245b)+10b

;Build 15-d coefficient matrix (per epoch) pixel by pixel, i.e.:(i,j) pair by (i,j) pair.
for i = 0, num_X_pixels-1 do begin
   for j = 0, num_Y_pixels-1 do begin
;     ;normalize vector of (i,j) pixel values by exposure.
      dep_vector  = reform(sfcs_in(i,j,*)/gt_expdur(index_in),n_elements(index_in))
      matrix_vector=regress(indep_vector, dep_vector ,weights,yfit,const,sigma,/RELATIVE_WEIGHT)
      matrix_vector=reform(matrix_vector,14)
      ;assign (cumulative) constant coefficient from fit. 
      matrix(0).coeff_vect(i,j) = const
      ;assign remaining coefficients for the indep_vector paramaters..
      for  k =1, 14 DO matrix(k).coeff_vect(i,j)= matrix_vector(k-1);magic number
   endfor
endfor

epoch_bound = [get_yo_dates(/entrance),ut_time(!stime)]
IF (keyword_set(ep_date)) THEN epoch_date = epoch_date ELSE $
    epoch_date=anytim(index_in(0),/yohkoh)

;TODO!
;--this needs to be improved (epoch_date determination) 
;do error correction, in the case that
;not all of the index entries are from the same epoch!!!!!

t_compare = ssw_time_compare(epoch_bound, epoch_date, /later)
later     = where((t_compare eq 1), num_gd_later)
epoch= later(0)-1


;obsolete, delete in future.
;fname=('synsfc_' + $
;       str_replace(strtrim(gt_filtb(index_in,/string),2),'.','') + $
;       time2file((get_yo_dates(/ent))(epoch)))(0)

outdir = ''
IF (1-keyword_set(out_file)) THEN BEGIN
  outdir=get_logenv('DIR_SXT_SYNSFC')
  fname=('synsfc_' + $
       str_replace(strtrim(gt_filta(index_in,/short),2),'.','') + $
       str_replace(strtrim(gt_filtb(index_in,/short),2),'.','') + $
       time2file((get_yo_dates(/ent))(epoch)))(0)
ENDIF ELSE fname = out_file

coeff_string = fname
savegenx, matrix, simplex, coeff_string,index_in, boxnote, file=concat_dir(outdir,fname), /overwrite

end

